﻿using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class Backdrop : Component
    {
        const int StarCount = 52;
        const int PointCount = 52;

        TVMiniMesh stars, points;

        public Backdrop(ICore core) : base(core) {}

        public override int Order
        {
            get { return -1; }
        }

        public override void Initialize()
        {
            TVMesh template = SpriteFactory.CreateSprite("Particle1");
            template.SetScale(0.2f, 0.2f, 1);
            stars = Scene.CreateMiniMesh(StarCount);
            stars.CreateFromMesh(template);
            stars.SetColor(Globals.RGBA(1, 1, 1, 0.6f));
            stars.SetColorMode(true);
            for (int i = 0; i < StarCount; i++)
            {
                stars.EnableMiniMesh(i, true);
                float distance = RandomHelper.Between(75, 0);
                float factor = MathHelper.Clamp(distance / 17, 1, 10);
                stars.SetPosition(RandomHelper.Centered(LevelState.Size.x * factor), RandomHelper.Centered(LevelState.Size.y * factor), distance, i);
            }

            template = SpriteFactory.CreateSprite("Particle6");
            template.SetScale(0.2f, 0.2f, 1);
            points = Scene.CreateMiniMesh(PointCount);
            points.CreateFromMesh(template);
            points.SetColor(Globals.RGBA(1, 1, 1, 0.5f));
            points.SetColorMode(true);
            for (int i = 0; i < PointCount; i++)
            {
                points.EnableMiniMesh(i, true);
                float distance = RandomHelper.Between(75, 0);
                float factor = MathHelper.Clamp(distance / 17, 1, 10);
                points.SetPosition(RandomHelper.Centered(LevelState.Size.x * factor), RandomHelper.Centered(LevelState.Size.y * factor), distance, i);
            }
        }

        public override void Draw()
        {
            points.Render();
            stars.Render();
        }

        [ServiceDependency]
        public ILevelState LevelState { private get; set; }
    }
}
