﻿using System.Collections.Generic;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class Borders : Component
    {
        const float CornerSize = 0.25f;

        public Borders(ICore core) : base(core) {}

        readonly List<TVMesh> meshes = new List<TVMesh>();

        public override void Initialize()
        {
            var mesh = SpriteFactory.CreateSprite("contour", new TV_COLOR(0.25f, 1, 0.25f, 0.15f));
            mesh.SetPosition(-LevelState.Size.x / 2, 0, 0);
            mesh.SetScale(0.1f, LevelState.Size.y - CornerSize * 2, 1);
            mesh.SetTextureModEnable(true);
            mesh.SetTextureModTranslationScale(0, 0, 0, 17.5f);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetPosition(LevelState.Size.x / 2, 0, 0);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetScale(0.1f, LevelState.Size.x - CornerSize * 2, 1);
            mesh.SetTextureModTranslationScale(0, 0, 0, 17.5f / LevelState.Size.y * LevelState.Size.x);
            mesh.SetRotation(0, 0, MathHelper.HalfPi);
            mesh.SetPosition(0, LevelState.Size.y / 2, 0);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetScale(0.1f, LevelState.Size.x - 1, 1);
            mesh.SetRotation(0, 0, -MathHelper.HalfPi);
            mesh.SetPosition(0, -LevelState.Size.y / 2, 0);
            meshes.Add(mesh);

            mesh = SpriteFactory.CreateSprite("corner", new TV_COLOR(0.25f, 1, 0.25f, 0.25f));
            mesh.SetPosition(-LevelState.Size.x / 2, LevelState.Size.y / 2, 0);
            mesh.SetScale(CornerSize * 1.5f, CornerSize * 1.5f, 1);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetPosition(-LevelState.Size.x / 2, -LevelState.Size.y / 2, 0);
            mesh.SetRotation(0, 0, MathHelper.HalfPi);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetPosition(LevelState.Size.x / 2, LevelState.Size.y / 2, 0);
            mesh.SetRotation(0, 0, -MathHelper.HalfPi);
            meshes.Add(mesh);

            mesh = mesh.Duplicate();
            mesh.SetPosition(LevelState.Size.x / 2, -LevelState.Size.y / 2, 0);
            mesh.SetRotation(0, 0, MathHelper.Pi);
            meshes.Add(mesh);
        }

        public override void Draw()
        {
            foreach (var mesh in meshes)
                mesh.Render();
        }

        [ServiceDependency]
        public ILevelState LevelState { private get; set; }
    }
}
