﻿using System;
using System.Linq;
using ComponentFramework;
using ComponentFramework.Modules;
using TroubleInEuclidea.Components.Waves;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Enemies
{
    [AutoLoad]
    class Enemies : Component
    {
        public Enemies(ICore core) : base(core) {}

        public override void Initialize()
        {
            GameState.Difficulty = DifficultyLevel.Noob;
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (GameState.Paused) return;

            if (PlayerState.Alive)
            {
                GameState.DifficultyTimer += elapsedTime;
                GameState.EnemiesTimer += elapsedTime;
            }

            // Upgrade difficulty?
            if (GameState.DifficultyTimer.TotalSeconds >= GameState.Difficulty.GetDuration())
                GameState.UpgradeDifficulty();

            // Time to spawn a wave?
            if (GameState.EnemiesTimer.TotalSeconds > GameState.Difficulty.GetWaveFrequency())
            {
                GameState.EnemiesTimer = TimeSpan.Zero;
                SpawnWave();
            }
        }

        void SpawnWave()
        {
            double probability = RandomHelper.Between(0, 1);
            var enemyTypes = GameState.Difficulty.GetAvailableEnemies();
            var occurenceSum = enemyTypes.Sum(x => x.GetOccurence());
            double acc = 0;
            EnemyType enemyType = EnemyType.Parallelogram;
            bool found = false;
            foreach (var type in enemyTypes)
            {
                acc += type.GetOccurence() / occurenceSum;
                if (acc > probability)
                {
                    enemyType = type;
                    found = true;
                    break;
                }
            }
            if (!found)
                throw new InvalidOperationException();

            // DEBUG
            //enemyType = EnemyType.Octogon;

            probability = RandomHelper.Between(0, 1);
            var waveTypes = enemyType.GetSupportedWaveTypes().Intersect(GameState.Difficulty.GetAvailableWaves());
            occurenceSum = waveTypes.Sum(x => x.GetOccurence());
            acc = 0;
            WaveType waveType = WaveType.Cluster;
            found = false;
            foreach (var type in waveTypes)
            {
                acc += type.GetOccurence() / occurenceSum;
                if (acc > probability)
                {
                    waveType = type;
                    found = true;
                    break;
                }
            }
            if (!found)
                throw new InvalidOperationException();

            var waveClass = waveType.GetClass(enemyType);
            var wave = Core.CreateAndLoadComponent(waveClass) as IWave;
            wave.Count = (int)Math.Ceiling(enemyType.GetMaximumWaveSize() * waveType.GetWaveCountModifier() / (6.0f - (int)GameState.Difficulty));

            SoundManager.Play(SoundManager.Load(enemyType.GetSpawnSound(waveType)));
        }

        [ServiceDependency]
        public IEnemiesState EnemiesState { private get; set; }
        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
        [ServiceDependency]
        public ISoundManager SoundManager { private get; set; }
    }
}