﻿using System;
using ComponentFramework;
using MTV3D65;
using ComponentFramework.Modules;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Enemies
{
    abstract class Enemy : Component, IShip
    {
        const float SpawnTime = 0.75f;

        bool flash;
        TimeSpan sinceSpawned;
        
        protected Enemy(ICore core) : base(core) {}

        public override void Initialize()
        {
            Mesh = SpriteFactory.CreateSprite(Texture, Color);
            Mesh.SetScale(Size.x, Size.y, 1);
        }

        public override sealed void Update(TimeSpan elapsedTime)
        {
            if (GameState.Paused) return;

            sinceSpawned += elapsedTime;

            if (!Ready)
                flash = Math.Round(sinceSpawned.TotalMilliseconds / 150) % 2 == 0;

            if (sinceSpawned.TotalSeconds > MovementDelay)
            {
                Act(elapsedTime);
                UpdatePosition();
            }

            Mesh.SetPosition(Position.x, Position.y, 0);
            Mesh.SetRotation(0, 0, (float)Theta);
        }

        void UpdatePosition()
        {
            CollisionResponse response = CollisionManager.CollideEllipses(Position, Size, Velocity, this);
            if (response.Collided)
                Velocity += response.Response;

            TV_2DVECTOR beforeImpulse = Position;
            Position += Velocity;
            Position = Position.Clamp(LevelState.Size / 2 - Size / 2);
            Velocity = Position - beforeImpulse;
        }

        protected abstract void Act(TimeSpan elapsedTime);

        public override void Draw()
        {
            Scene.SetTextureFilter(CONST_TV_TEXTUREFILTER.TV_FILTER_NEARESTPOINT);

            if (flash || Ready) 
                Mesh.Render();
        }

        protected override void DisposeInternal()
        {
            Mesh.Destroy();
        }

        public abstract EnemyType Type { get; }
        public abstract TV_2DVECTOR Size { get; }
        protected abstract string Texture { get; }
        public abstract TV_COLOR Color { get; }

        protected TVMesh Mesh { get; private set; }

        public double Theta { get; protected set; }
        public TV_2DVECTOR Position { get; internal set; }
        public TV_2DVECTOR Velocity { get; set; }

        public bool Ready { get { return sinceSpawned.TotalSeconds > SpawnTime; } }
        public float MovementDelay { get; set; }

        public abstract int Points { get; }

        [ServiceDependency]
        public ICollisionManager CollisionManager { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
        [ServiceDependency]
        public ILevelState LevelState { private get; set; }
    }
}