﻿using System;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Enemies
{
    class OctogonEnemy : Enemy
    {
        const float TerminalVelocity = 2;
        const float Acceleration = 0.03f;
        const float RotateSpeed = 0.1f;
        TimeSpan accumulator;

        public OctogonEnemy(ICore core) : base(core) {}

        public override void Initialize()
        {
            base.Initialize();
            accumulator = TimeSpan.FromSeconds(RandomHelper.Between(0, MathHelper.TwoPi));
        }

        protected override void Act(TimeSpan elapsedTime)
        {
            TV_2DVECTOR difference = PlayerState.Position - Position;
            float distance = difference.Length();
            TV_2DVECTOR direction = distance == 0 ? Vector2.Zero : difference / distance;

            accumulator += elapsedTime;
            if (accumulator.TotalSeconds > MathHelper.TwoPi)
                accumulator -= TimeSpan.FromSeconds(MathHelper.TwoPi);

            Velocity += new TV_2DVECTOR((float)Math.Cos(accumulator.TotalSeconds * 3), (float)Math.Sin(accumulator.TotalSeconds * 3)) * (float)elapsedTime.TotalSeconds * RotateSpeed;

            if (PlayerState.Alive)
                Velocity += new TV_2DVECTOR(direction.x, direction.y) * Acceleration * (float)elapsedTime.TotalSeconds;

            Velocity = Velocity.Clamp(new TV_2DVECTOR(TerminalVelocity, TerminalVelocity) * (float)elapsedTime.TotalSeconds);
        }

        public override EnemyType Type
        {
            get { return EnemyType.Octogon; }
        }

        public override TV_2DVECTOR Size
        {
            get { return new TV_2DVECTOR(0.9f, 0.9f); }
        }

        public override TV_COLOR Color
        {
            get { return new TV_COLOR(0.8049f, 0.08549f, 0.15f, 1); }
        }

        public override int Points
        {
            get { return 75; }
        }

        protected override string Texture
        {
            get { return "Octogon"; }
        }

        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
    }
}
