﻿using System;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Enemies
{
    class ParallelogramEnemy : Enemy
    {
        const float ProximityThreshold = 1;
        const float Acceleration = 0.03f;
        const float TerminalVelocity = 2;

        bool close;
        int closeTex, farTex;

        public ParallelogramEnemy(ICore core) : base(core) {}

        public override void Initialize()
        {
            base.Initialize();

            closeTex = TextureFactory.LoadTexture("Parallelogram Close.dds");
            farTex = Globals.GetTex(Texture);
        }

        protected override void Act(TimeSpan elapsedTime)
        {
            TV_2DVECTOR difference = PlayerState.Position - Position;
            float distance = difference.Length();
            TV_2DVECTOR direction = distance == 0 ? Vector2.Zero : difference / distance;
            float proximity = Math.Max(12 / Math.Min(Math.Max(distance, 3), 12) / 3, 1);

            bool nowClose = proximity > ProximityThreshold;
            if (!close && nowClose)
                Mesh.SetTexture(closeTex);
            else if (close && !nowClose)
                Mesh.SetTexture(farTex);
            close = nowClose;

            if (PlayerState.Alive)
                Velocity += new TV_2DVECTOR(direction.x, direction.y) * Acceleration * (float)elapsedTime.TotalSeconds * proximity;
            Velocity = Velocity.Clamp(new TV_2DVECTOR(TerminalVelocity, TerminalVelocity) * (float)elapsedTime.TotalSeconds * proximity);
        }

        public override EnemyType Type
        {
            get { return EnemyType.Parallelogram; }
        }

        public override TV_2DVECTOR Size
        {
            get { return new TV_2DVECTOR(0.6f, 0.9f); }
        }

        protected override string Texture
        {
            get { return "Parallelogram Far"; }
        }

        public override TV_COLOR Color
        {
            get { return new TV_COLOR(0.15f, 0.2f, 1, 1); }
        }

        public override int Points
        {
            get { return 25; }
        }

        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
    }
}
