﻿using System;
using ComponentFramework;
using MTV3D65;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Enemies
{
    class TriangleEnemy : Enemy
    {
        const float ImpulseSpeed = 0.1f;
        const float Friction = 0.99f;
        const float MinStreakDuration = 0.5f;
        const float MaxStreakDuration = 3;

        double streakTimeLeft;
        float destinationTheta;

        public TriangleEnemy(ICore core) : base(core) {}

        protected override void Act(TimeSpan elapsedTime)
        {
            if (streakTimeLeft <= 0)
            {
                streakTimeLeft = RandomHelper.Between(MinStreakDuration, MaxStreakDuration);
                var change = new TV_2DVECTOR(RandomHelper.Centered(ImpulseSpeed), RandomHelper.Centered(ImpulseSpeed));
                Velocity += change.Lerp((Position * -1).Normalize() * ImpulseSpeed, RandomHelper.Between(0, 1));
                destinationTheta = MathHelper.WrapAngle(MathHelper.Pi - MathHelper.LookAt(Velocity.y, Velocity.x, Direction.Up));
            }

            Velocity *= Friction;
            Theta = MathHelper.CurveAngle(Theta, destinationTheta, 0.1f);

            streakTimeLeft -= elapsedTime.TotalSeconds;
        }

        public override EnemyType Type
        {
            get { return EnemyType.Triangle; }
        }

        public override TV_2DVECTOR Size
        {
            get { return new TV_2DVECTOR(0.75f, 0.75f); }
        }

        protected override string Texture
        {
            get { return "Triangle"; }
        }

        public override TV_COLOR Color
        {
            get { return new TV_COLOR(1, 0.15f, 0.05f, 1); }
        }

        public override int Points
        {
            get { return 50; }
        }
    }
}