﻿using System;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class Exhaust : Component
    {
        const int ParticleTypes = 7;

        float colorWheel;
        TVParticleSystem particles;

        public Exhaust(ICore core) : base(core) {}

        public override void Initialize()
        {
            particles = Scene.CreateParticleSystem();

            for (int i = 0; i < ParticleTypes; i++)
            {
                particles.CreateEmitter(CONST_TV_EMITTERTYPE.TV_EMITTER_POINTSPRITE, 100);
                particles.SetPointSprite(i, TextureFactory.LoadTexture("Particle" + i + ".dds"), 0.175f, true, 0.175f);
                particles.SetEmitterAlphaBlending(i, CONST_TV_PARTICLECHANGE.TV_CHANGE_ALPHA, CONST_TV_BLENDINGMODE.TV_BLEND_ALPHA);
                particles.SetEmitterAlphaTest(i, false);
                particles.SetEmitterSpeed(i, 70 + i * 7);
                particles.SetEmitterPower(i, 0.6f, 0.4f);
                particles.SetEmitterShape(i, CONST_TV_EMITTERSHAPE.TV_EMITTERSHAPE_SPHEREVOLUME);
                particles.SetEmitterSphereRadius(i, 0.1f);
            }
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (GameState.Paused) return;

            for (int i = 0; i < ParticleTypes; i++)
            {
                float xAdd = (float)Math.Cos(PlayerState.Theta + MathHelper.Pi) * 0.4f, yAdd = (float)Math.Sin(PlayerState.Theta + MathHelper.Pi) * 0.4f;
                particles.SetEmitterPosition(i, new TV_3DVECTOR(PlayerState.Position.x + xAdd, PlayerState.Position.y + yAdd, 0));

                particles.SetEmitterDirection(i, true, new TV_3DVECTOR(-PlayerState.Velocity.x, -PlayerState.Velocity.y, 0), new TV_3DVECTOR(0.5f, 0.5f, 0));

                float l = PlayerState.Velocity.Abs().Dot(Vector2.One) > 0.01f ? 1 : 0;
                if (!PlayerState.Alive)
                    l = 0;

                colorWheel += (float)elapsedTime.TotalSeconds;
                if (colorWheel > 3) colorWheel -= 3;
                float r = MathHelper.Clamp(1 - Math.Abs(colorWheel - 1) + 0.3f, 0, 1);
                float g = MathHelper.Clamp(1 - Math.Abs(colorWheel - 2) + 0.3f, 0, 1);
                float b = MathHelper.Clamp(1 - Math.Abs(colorWheel - 3) + 0.3f, 0, 1);
                particles.SetParticleDefaultColor(i, new TV_COLOR(r * l, g * l, b * l, 1));
            }

            particles.Update();
        }

        public override void Draw()
        {
            particles.Render();
        }

        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
    }
}
