﻿using ComponentFramework;
using ComponentFramework.Modules;
using System;
using MTV3D65;
using TroubleInEuclidea.Components.Waves;
using TroubleInEuclidea.Structure;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Components.Enemies;

namespace TroubleInEuclidea.Components
{
    class ExplodingShip : Component
    {
        const int ParticleTypes = 5;

        readonly IShip ship;
        TimeSpan pastTime;
        TVParticleSystem particles;

        public ExplodingShip(ICore core, IShip ship) : base(core)
        {
            this.ship = ship;
        }

        public override void Initialize()
        {
            particles = Scene.CreateParticleSystem();

            for (int i = 0; i < ParticleTypes; i++)
            {
                int amount = ship is Player ? 20 : 3;
                float power = ship is Player ? RandomHelper.Between(1, 4) : RandomHelper.Between(0.5f, 1.5f);

                particles.CreateEmitter(CONST_TV_EMITTERTYPE.TV_EMITTER_BILLBOARD, amount);
                particles.SetBillboard(i, TextureFactory.LoadTexture("Fragment" + i + ".dds"), 0.25f, 0.25f);
                particles.SetEmitterAlphaBlending(i, CONST_TV_PARTICLECHANGE.TV_CHANGE_NO, CONST_TV_BLENDINGMODE.TV_BLEND_ALPHA);
                particles.SetEmitterAlphaTest(i, false);
                particles.SetEmitterSpeed(i, i);
                particles.SetEmitterPower(i, power, 1.5f);
                particles.SetEmitterShape(i, CONST_TV_EMITTERSHAPE.TV_EMITTERSHAPE_SPHEREVOLUME);
                particles.SetEmitterSphereRadius(i, (ship.Size.x + ship.Size.y) / 3);
                particles.SetEmitterPosition(i, ship.Position.ToVector3());
                particles.SetEmitterDirection(i, true, ship.Velocity.Normalize().ToVector3(), new TV_3DVECTOR(0.75f, 0.75f, 0));

                var keyframes = new[]
                {
                    new TV_PARTICLE_KEYFRAME { fKey = 0, cColor = new TV_COLOR(ship.Color.r, ship.Color.g, ship.Color.b, 1), fSize = new TV_3DVECTOR(1, 1, 1), vRotation = new TV_3DVECTOR(0, 0, 0) },
                    new TV_PARTICLE_KEYFRAME { fKey = 1.5f, cColor = new TV_COLOR(ship.Color.r, ship.Color.g, ship.Color.b, 0), fSize = new TV_3DVECTOR(1, 1, 1), vRotation = new TV_3DVECTOR(0, 0, RandomHelper.Centered(MathHelper.TwoPi)) }
                };
                particles.SetParticleKeyFrames(i, (int)(CONST_TV_PARTICLE_KEYUSAGE.TV_PARITCLE_KEYUSAGE_ANGLE | CONST_TV_PARTICLE_KEYUSAGE.TV_PARTICLE_KEYUSAGE_COLOR), 2, keyframes);
            }

            if (ship is ParallelogramEnemy)
                SoundManager.Play(SoundManager.Load("boom2"), false);
            else if (ship is TriangleEnemy)
                SoundManager.Play(SoundManager.Load("boom3"), false);
            else if (ship is OctogonEnemy)
            {
                SoundManager.Play(SoundManager.Load("buzz11"), false);
                var wave = Core.CreateAndLoadComponent<ClusterWave<SquareEnemy>>();
                wave.Count = 4;
                wave.Center = ship.Position;
            } 
            else if (ship is SquareEnemy)
                SoundManager.Play(SoundManager.Load("boom4"), false);
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (GameState.Paused) return;

            pastTime += elapsedTime;

            if (pastTime.TotalSeconds >= 1.5)
                Core.UnloadComponent(this);
            else
                particles.Update();
        }

        protected override void DisposeInternal()
        {
            particles.Destroy();
        }

        public override void Draw()
        {
            particles.Render();
        }

        [ServiceDependency]
        public ISoundManager SoundManager { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
    }
}