﻿using System;
using System.Windows.Forms;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Services;
using System.Drawing;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class GameStateControl : Component
    {
        readonly TV_KEYDATA[] data = new TV_KEYDATA[256];
        int pause;

        public GameStateControl(ICore core) : base(core) {}

        public override void Initialize()
        {
            base.Initialize();
            pause = TextureFactory.LoadTexture("pause.dds");
        }

        public override int Order
        {
            get { return 500; }
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (PlayerState.Lives > 0)
            {
                int keys = 0;
                InputEngine.GetKeyBuffer(data, ref keys);
                for (int i = 0; i < keys; i++)
                {
                    var key = data[i];
                    if (key.Key == (int)CONST_TV_KEY.TV_KEY_RETURN && key.Pressed != 0)
                    {
                        GameState.Paused = !GameState.Paused;

                        if (GameState.Paused)
                        {
                            Cursor.Show();
                            SoundManager.Paused = true;
                        }
                        else
                        {
                            Cursor.Hide();
                            SoundManager.Paused = false;
                        }
                    }
                }
            }

            if (!GameState.Paused)
                Cursor.Clip = Core.Settings.WindowedForm.RectangleToScreen(Core.Settings.WindowedForm.ClientRectangle);
            else
                Cursor.Clip = Rectangle.Empty;

            if (InputEngine.IsKeyPressed(CONST_TV_KEY.TV_KEY_ESCAPE))
                Application.Exit();
        }

        public override void Draw()
        {
            if (GameState.Paused)
            {
                int width = Core.Engine.GetViewport().GetWidth();
                int height = Core.Engine.GetViewport().GetHeight();

                Screen2DImmediate.Draw_FilledBox(0, 0, width - 1, height - 1, Globals.RGBA(0, 0, 0, 0.5f));

                int x = width / 2 - 63 / 2;
                int y = height / 2 - 28 / 2;
                Screen2DImmediate.Draw_Texture(pause, x, y, x + 63 - 1, y + 28 - 1, Globals.RGBA(0.25f, 1, 0.25f, 1));
            }
        }

        [ServiceDependency]
        public ISoundManager SoundManager { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
    }
}
