﻿using ComponentFramework;
using ComponentFramework.Modules;
using TroubleInEuclidea.Services;
using System;
using TroubleInEuclidea.Structure;
using System.Xml.Serialization;
using System.IO;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class HeadsUpDisplay : Component
    {
        struct NumberDescriptor
        {
            public int Width;
            public int TextureId;
        }

        int star, ship, rank, lives, score, pressSpace, splash, hi;
        NumberDescriptor[] numbers;
        TimeSpan accumulator;
        HiScore hiScore;
        readonly XmlSerializer serializer = new XmlSerializer(typeof(HiScore));

        public HeadsUpDisplay(ICore core) : base(core) {}

        public override void Initialize()
        {
            if (File.Exists("HiScore.xml"))
                using (var reader = new StreamReader("HiScore.xml"))
                {
                    hiScore = serializer.Deserialize(reader) as HiScore;
                }
            else
                hiScore = new HiScore();

            star = TextureFactory.LoadTexture("Star.dds");
            ship = TextureFactory.LoadTexture("Ship.dds");
            rank = TextureFactory.LoadTexture("Rank.dds");
            lives = TextureFactory.LoadTexture("Ships.dds");
            score = TextureFactory.LoadTexture("Score.dds");
            pressSpace = TextureFactory.LoadTexture("PressSpace.dds");
            splash = TextureFactory.LoadTexture("Splash.dds");
            hi = TextureFactory.LoadTexture("Hi.dds");

            numbers = new NumberDescriptor[10];
            for(int i = 0; i <= 9; i++)
            {
                numbers[i] = new NumberDescriptor { TextureId = TextureFactory.LoadTexture(i + ".dds") };
                switch (i)
                {
                    case 0: case 4: case 6:
                    case 7: case 8: case 9:
                        numbers[i].Width = 13;
                        break;
                    case 2: case 5:
                        numbers[i].Width = 12;
                        break;
                    case 3:
                        numbers[i].Width = 15;
                        break;
                    case 1:
                        numbers[i].Width = 3;
                        break;
                }
            }
        }

        public override void Update(TimeSpan elapsedTime)
        {
            accumulator += elapsedTime;
            if (accumulator.TotalSeconds > MathHelper.TwoPi)
                accumulator -= TimeSpan.FromSeconds(MathHelper.TwoPi);

            if (PlayerState.Lives == 0)
            {
                if (PlayerState.Score > hiScore.Score)
                {
                    hiScore.Score = PlayerState.Score;
                    using (var writer = new StreamWriter("HiScore.xml"))
                    {
                        serializer.Serialize(writer, hiScore);
                    }
                }
            }
        }

        public override void Draw()
        {
            Screen2DImmediate.Action_Begin2D();
            int x = 5, y = 5;

            Screen2DImmediate.Draw_Texture(rank, x, y, x + 181 - 1, y + 28 - 1, Globals.RGBA(0.25f, 1, 0.25f, 1));

            x += 181 - 1 + 5;
            for (int i = 1; i <= (int)GameState.Difficulty + 1; i++)
            {
                Screen2DImmediate.Draw_Texture(star, x, y, x + 19 - 1, y + 27 - 1, Globals.RGBA(1, 0.992f, 0.49f, 1));
                x += 19 - 1 + 5;
            }

            int width = Core.Engine.GetViewport().GetWidth();
            x = width;

            for (int i = 0; i < PlayerState.Lives; i++)
            {
                x -= 5 + 24;
                Screen2DImmediate.Draw_Texture(ship, x, y + 5, x + 24 - 1, y + 21 - 1 + 5, Globals.RGBA(1, 0.992f, 0.49f, 1));
            }
            x -= 67 + 5;
            Screen2DImmediate.Draw_Texture(lives, x, y, x + 67 - 1, y + 28 - 1, Globals.RGBA(0.25f, 1, 0.25f, 1));

            int height = Core.Engine.GetViewport().GetHeight();
            y = height - 5;
            x = 5;
            Screen2DImmediate.Draw_Texture(score, x, y - 25 - 1, x + 67 - 1, y, Globals.RGBA(0.25f, 1, 0.25f, 1));
            x += 67 - 1 + 5;

            foreach (char c in PlayerState.Score.ToString())
            {
                int digit = int.Parse(c.ToString());
                Screen2DImmediate.Draw_Texture(numbers[digit].TextureId, x, y - 24 - 1, x + numbers[digit].Width, y, Globals.RGBA(1, 0.992f, 0.49f, 1));
                x += numbers[digit].Width + 2;
            }

            x += 20;

            Screen2DImmediate.Draw_Texture(hi, x, y - 28 - 1, x + 31 - 1, y, Globals.RGBA(0.25f, 1, 0.25f, 1));
            x += 31 - 1 + 5;
            foreach (char c in hiScore.Score.ToString())
            {
                int digit = int.Parse(c.ToString());
                Screen2DImmediate.Draw_Texture(numbers[digit].TextureId, x, y - 24 - 1, x + numbers[digit].Width, y, Globals.RGBA(1, 0.992f, 0.49f, 1));
                x += numbers[digit].Width + 2;
            }

            if (PlayerState.Lives == 0)
            {
                x = width / 2 - 237 / 2;
                y = height / 2 - (138 + 28) / 2 - 25;
                Screen2DImmediate.Draw_Texture(splash, x, y, x + 237 - 1, y + 138 - 1, Globals.RGBA(0.25f, 1, 0.25f, 1));

                x = width / 2 - 158 / 2;
                y += 138 + 20;
                Screen2DImmediate.Draw_Texture(pressSpace, x, y, x + 158 - 1, y + 28 - 1, Globals.RGBA(0.25f, 1, 0.25f, (float)Math.Cos(accumulator.TotalSeconds * 4) / 2 + 0.5f));
            }

            Screen2DImmediate.Action_End2D();
        }

        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
        [ServiceDependency]
        public IGameState GameState { private get; set; }
    }
}
