﻿using System;
using System.Collections.Generic;
using ComponentFramework;
using ComponentFramework.Modules;
using TroubleInEuclidea.Services;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components
{
    [AutoLoad]
    class PowerUps : Component
    {
        enum Accomplishment { None, Nonviolence, Korean }
        class AccomplishmentState
        {
            public int Texture;
            public bool Complete;
        }

        Dictionary<Accomplishment, AccomplishmentState> accomplishments;
        TimeSpan timeComplete;
        Accomplishment current;
        int lastScore;

        public PowerUps(ICore core) : base(core) {}

        public override void Initialize()
        {
            accomplishments = new Dictionary<Accomplishment, AccomplishmentState>
            {{
                Accomplishment.Nonviolence,
                new AccomplishmentState { Texture = TextureFactory.LoadTexture("nonviolence.dds") } 
            }, {
                Accomplishment.Korean,
                new AccomplishmentState { Texture = TextureFactory.LoadTexture("korean.dds") } 
            }};
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (PlayerState.Score > lastScore + 10000)
            {
                PlayerState.Lives++;
                SoundManager.Play(SoundManager.Load("beep6"));
                lastScore += 10000;
            }

            if (GameState.Difficulty == DifficultyLevel.Addict && PlayerState.Score == 0 && !accomplishments[Accomplishment.Nonviolence].Complete)
            {
                SoundManager.Play(SoundManager.Load("beep7"));
                timeComplete = TimeSpan.Zero;
                current = Accomplishment.Nonviolence;
                accomplishments[current].Complete = true;
            }

            if (GameState.Difficulty == DifficultyLevel.Korean && PlayerState.HasNotDied && !accomplishments[Accomplishment.Korean].Complete)
            {
                SoundManager.Play(SoundManager.Load("beep7"));
                timeComplete = TimeSpan.Zero;
                current = Accomplishment.Korean;
                accomplishments[current].Complete = true;
            }

            if (current != Accomplishment.None)
            {
                timeComplete += elapsedTime;
                if (timeComplete.TotalSeconds > 4)
                    current = Accomplishment.None;
            }

#if DEBUG
            if (InputEngine.IsKeyPressed(MTV3D65.CONST_TV_KEY.TV_KEY_MINUS))
                PlayerState.Score = 5000;
#endif
        }

        public override void Draw()
        {
            if (current != Accomplishment.None)
            {
                int width = Core.Engine.GetViewport().GetWidth();
                int height = Core.Engine.GetViewport().GetHeight();

                int color = Globals.RGBA(0.25f, 1, 0.25f, MathHelper.Lerp(1, 0, (float)timeComplete.TotalSeconds / 4));

                int x = width / 2 - 128;
                int y = height - 5 - 64;

                Screen2DImmediate.Draw_Texture(accomplishments[current].Texture, x, y, x + 256 - 1, y + 64 - 1, color);
            }
        }

        [ServiceDependency]
        public IGameState GameState { private get; set; }
        [ServiceDependency]
        public IPlayerState PlayerState { private get; set; }
        [ServiceDependency]
        public ISoundManager SoundManager { private get; set; }
    }
}
