﻿using System;
using ComponentFramework;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Waves
{
    class BurstWave<T> : Wave<T> where T : Enemy
    {
        const float Padding = 1;
        const float Frequency = 0.4f;

        int SpawnCount { get; set; }

        public Direction Horizontal { get; set; }
        public Direction Vertical { get; set; }

        TimeSpan TimeAccumulator { get; set; }

        public BurstWave(ICore core) : base(core) { }

        protected override void CreateEnemies()
        {
            if (Horizontal == Direction.None)
                Horizontal = RandomHelper.Probability(0.5) ? Direction.Left : Direction.Right;
            if (Vertical == Direction.None)
                Vertical = RandomHelper.Probability(0.5) ? Direction.Up : Direction.Down;

            // Only create a single one at init time
            CreateAndPlace();
        }

        void CreateAndPlace()
        {
            var enemy = Core.CreateAndLoadComponent<T>();
            EnemiesState.Enemies.Add(enemy);
            enemy.Position = (Horizontal.AsVector() + Vertical.AsVector())
                    .ComponentWiseProduct(LevelState.Size / 2 - Padding * Vector2.One);

            SpawnCount++;
        }

        protected override void PlaceEnemies()
        {
            // Nothing to do here
        }

        public override void Update(TimeSpan elapsedTime)
        {
            TimeAccumulator += elapsedTime;

            if (TimeAccumulator.TotalSeconds > Frequency)
            {
                TimeAccumulator -= TimeSpan.FromSeconds(Frequency);
                CreateAndPlace();

                if (SpawnCount >= WaveEnemies.Length)
                    DieOut();
            }
        }
    }
}