﻿using ComponentFramework;
using MTV3D65;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Structure;
using System;

namespace TroubleInEuclidea.Components.Waves
{
    class ClusterWave<T> : Wave<T> where T : Enemy
    {
        readonly static TV_2DVECTOR ClusterDensity = new TV_2DVECTOR(1, 1);

        public TV_2DVECTOR? Center { get; set; }

        public ClusterWave(ICore core) : base(core) { }

        protected override void PlaceEnemies()
        {
            var rows = (int)Math.Round(Math.Sqrt(WaveEnemies.Length));
            var cols = (int)Math.Ceiling(WaveEnemies.Length / (double)rows);

            TV_2DVECTOR enemySize = WaveEnemies[0].Size;

            bool useDelay = true;
            if (Center == null)
            {
                TryPlace(WaveEnemies[0], () => new TV_2DVECTOR(
                                                       RandomHelper.Centered(LevelState.Size.x / 2 - enemySize.x * cols / 2),
                                                       RandomHelper.Centered(LevelState.Size.y / 2 - enemySize.y * rows / 2)));
                Center = WaveEnemies[0].Position;
            }
            else
                useDelay = false;

            for (int i = 0; i < rows; i++)
                for (int j = 0; j < cols; j++)
                {
                    int enemyIndex = j + i * cols;
                    if (enemyIndex == WaveEnemies.Length)
                        return;

                    TV_2DVECTOR position;
                    position.x = (cols / 2f - j + (i % 2) * 0.5f) * enemySize.x / ClusterDensity.x;
                    position.y = (rows / 2f - i) * enemySize.y / ClusterDensity.y;

                    WaveEnemies[enemyIndex].Position = Center.Value + position;
                    WaveEnemies[enemyIndex].MovementDelay = useDelay ? 1 : 0.25f;
                }
        }
    }
}