﻿using System;
using ComponentFramework;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Components.Waves;
using TroubleInEuclidea.Structure;

namespace TroubleInEuclidea.Components.Waves
{
    class SquareWave<T> : Wave<T> where T : Enemy
    {
        public SquareWave(ICore core) : base(core) { }

        protected override void CreateEnemies()
        {
            int enemiesBySide = WaveEnemies.Length / 4;
            float horizontalRatio = LevelState.Size.x / LevelState.Size.y;
            var horizontalCount = (int)Math.Round(enemiesBySide / horizontalRatio);
            var verticalCount = (int)Math.Round(enemiesBySide * horizontalRatio);

            Core.LoadComponent(new LinearWave<T>(Core) { Direction = Direction.Down, Count = verticalCount });
            Core.LoadComponent(new LinearWave<T>(Core) { Direction = Direction.Up, Count = verticalCount });
            Core.LoadComponent(new LinearWave<T>(Core) { Direction = Direction.Left, Count = horizontalCount });
            Core.LoadComponent(new LinearWave<T>(Core) { Direction = Direction.Right, Count = horizontalCount });
        }

        protected override void PlaceEnemies()
        {
            // Already placed by the sub-components
        }
    }
}