﻿using System;
using System.Diagnostics;
using ComponentFramework.Modules;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Services;
using ComponentFramework;
using MTV3D65;

namespace TroubleInEuclidea.Components.Waves
{
    abstract class Wave<T> : Component, IWave where T : Enemy
    {
        const float PlacementRetryLimit = 1 / 15f;

        protected T[] WaveEnemies { get; private set; }

        protected Wave(ICore core) : base(core) {}

        public int Count
        {
            set { WaveEnemies = new T[value]; }
        }

        public override sealed void Initialize()
        {
            CreateEnemies();
            PlaceEnemies();
        }

        protected virtual void CreateEnemies()
        {
            for (int i = 0; i < WaveEnemies.Length; i++)
                EnemiesState.Enemies.Add(WaveEnemies[i] = Core.CreateAndLoadComponent<T>());
        }

        protected abstract void PlaceEnemies();

        protected bool TryPlace(T enemy, Func<TV_2DVECTOR> placingFunction)
        {
            var watch = new Stopwatch();
            watch.Start();

            TV_2DVECTOR tentativePosition = placingFunction();
            while (CollisionManager.OverlapsAny(tentativePosition, enemy.Size, enemy) != null)
            {
                if (watch.Elapsed.TotalSeconds > PlacementRetryLimit)
                {
                    Core.UnloadComponent(enemy);
                    return false;
                }
                tentativePosition = placingFunction();
            }

            enemy.Position = tentativePosition;
            return true;
        }

        public override void Update(TimeSpan elapsedTime)
        {
            DieOut();
        }

        protected void DieOut()
        {
            Core.UnloadComponent(this);
        }

        public sealed override void Draw() { }
        public sealed override void PreDraw() { }
        public sealed override void PostDraw() { }

        [ServiceDependency]
        public ICollisionManager CollisionManager { protected get; set; }
        [ServiceDependency]
        public ILevelState LevelState { protected get; set; }
        [ServiceDependency]
        public IEnemiesState EnemiesState { protected get; set; }
    }

    interface IWave
    {
        int Count { set; }
    }
}