﻿using System;
using System.Linq;
using ComponentFramework;
using ComponentFramework.Modules;
using MTV3D65;
using TroubleInEuclidea.Structure;
using TroubleInEuclidea.Components.Enemies;

namespace TroubleInEuclidea.Services
{
    class CollisionManager : Service, ICollisionManager
    {
        const float CollisionPermissivity = 0.6f;

        public CollisionManager(ICore core) : base(core) {}

        public CollisionResponse CollideEllipses(TV_2DVECTOR origin, TV_2DVECTOR size, TV_2DVECTOR velocity)
        {
            return CollideEllipses(origin, size, velocity, null);
        }
        public CollisionResponse CollideEllipses(TV_2DVECTOR origin, TV_2DVECTOR size, TV_2DVECTOR velocity, Enemy exception)
        {
            var response = new CollisionResponse();
            TV_2DVECTOR destination = origin + velocity;

            foreach (Enemy collider in EnemiesState.Enemies.Where(x => x.Ready && (exception == null || (x != exception && x.Type == exception.Type))))
            {
                TV_2DVECTOR difference = destination - collider.Position;
                float distance = difference.Length();
                TV_2DVECTOR collisionNormal = distance == 0 ? Vector2.Zero : difference / distance;

                TV_2DVECTOR jointSize = (collider.Size + size) / 2 * CollisionPermissivity;
                float step = Math.Abs(collisionNormal.y);
                float jointRadius = MathHelper.Slerp(jointSize.x * Vector2.UnitX, jointSize.y * Vector2.UnitY, step).Length();

                if (jointRadius > distance)
                {
                    response.Response += (jointRadius - distance) * collisionNormal;
                    response.Collided = true;
                }
            }

            return response;
        }

        public Enemy OverlapsAny(TV_2DVECTOR origin, TV_2DVECTOR size)
        {
            return OverlapsAny(origin, size, null);
        }
        public Enemy OverlapsAny(TV_2DVECTOR origin, TV_2DVECTOR size, Enemy exception)
        {
            return EnemiesState.Enemies.FirstOrDefault(x =>
                (exception == null || (x != exception && x.Type == exception.Type)) &&
                x.Ready &&
                x.Position.AlmostEqual(origin, (x.Size + size) / 2 * CollisionPermissivity));
        }

        [ServiceDependency]
        public IEnemiesState EnemiesState { private get; set; }
    }

    interface ICollisionManager : IService
    {
        CollisionResponse CollideEllipses(TV_2DVECTOR origin, TV_2DVECTOR size, TV_2DVECTOR velocity);
        CollisionResponse CollideEllipses(TV_2DVECTOR origin, TV_2DVECTOR size, TV_2DVECTOR velocity, Enemy exception);

        Enemy OverlapsAny(TV_2DVECTOR origin, TV_2DVECTOR size);
        Enemy OverlapsAny(TV_2DVECTOR origin, TV_2DVECTOR size, Enemy exception);
    }
}
