﻿using System;
using ComponentFramework;
using ComponentFramework.Modules;
using IrrKlang;

namespace TroubleInEuclidea.Services
{
    class SoundManager : Service, ISoundManager
    {
        const string ContentPath = @"Content\";
        ISoundEngine Engine { get; set; }

        public SoundManager(ICore core) : base(core) {}

        public override void Initialize()
        {
            Engine = new ISoundEngine();
        }

        public ISoundSource Load(string name)
        {
            Console.WriteLine(name);
            return Engine.GetSoundSource(ContentPath + name + ".ogg", true);
        }

        public ISound Play(ISoundSource source)
        {
            return Play(source, false);
        }
        public ISound Play(ISoundSource source, bool loop)
        {
            return GetSound(source, loop, true);
        }
        public ISound GetSound(ISoundSource source, bool loop, bool play)
        {
            return Engine.Play2D(source, loop, !play, false);
        }

        public bool Paused 
        {
            set { Engine.SetAllSoundsPaused(value); }
        }

        protected override void DisposeInternal()
        {
            Engine.StopAllSounds();
            Engine.RemoveAllSoundSources();
        }
    }

    interface ISoundManager : IService
    {
        ISoundSource Load(string name);
        ISound Play(ISoundSource source);
        ISound Play(ISoundSource source, bool loop);
        ISound GetSound(ISoundSource source, bool loop, bool play);
        bool Paused { set; }
    }
}
