﻿using System;
using System.Collections.Generic;

namespace TroubleInEuclidea.Structure
{
    enum DifficultyLevel
    {
        Noob,
        Amateur,
        Skilled,
        Addict,
        Gentleman,
        Korean
    }

    static class DifficultyLevelExtensions
    {
        public static IEnumerable<EnemyType> GetAvailableEnemies(this DifficultyLevel difficulty)
        {
            if (difficulty >= DifficultyLevel.Noob)
                yield return EnemyType.Parallelogram;

            if (difficulty >= DifficultyLevel.Amateur)
                yield return EnemyType.Triangle;

            //if (difficulty >= DifficultyLevel.Skilled)
            //    yield return EnemyType.Circle;

            if (difficulty >= DifficultyLevel.Skilled)
                yield return EnemyType.Octogon;
        }

        public static IEnumerable<WaveType> GetAvailableWaves(this DifficultyLevel difficulty)
        {
            if (difficulty >= DifficultyLevel.Noob)
                yield return WaveType.Random;

            if (difficulty >= DifficultyLevel.Amateur)
            {
                yield return WaveType.Cluster;
                yield return WaveType.Linear;
            }

            if (difficulty >= DifficultyLevel.Skilled)
            {
                yield return WaveType.Burst;
                yield return WaveType.Square;
            }

            if (difficulty >= DifficultyLevel.Addict)
                yield return WaveType.SynchronousBurst;
        }

        public static double GetDuration(this DifficultyLevel difficulty)
        {
            switch (difficulty)
            {
                case DifficultyLevel.Noob: return 15;
                case DifficultyLevel.Amateur: return 20;
                case DifficultyLevel.Skilled: return 25;
                case DifficultyLevel.Addict: return 30;
                case DifficultyLevel.Gentleman: return 30;
                case DifficultyLevel.Korean: return double.MaxValue; // lol.
            }
            throw new InvalidOperationException();
        }

        public static double GetWaveFrequency(this DifficultyLevel difficulty)
        {
            switch (difficulty)
            {
                case DifficultyLevel.Noob: return 2.5f;
                case DifficultyLevel.Amateur: return 2;
                case DifficultyLevel.Skilled: return 2.5f;
                case DifficultyLevel.Addict: return 2;
                case DifficultyLevel.Gentleman: return 1.5f;
                case DifficultyLevel.Korean: return 1.75f;
            }
            throw new InvalidOperationException();
        }
    }
}
