﻿using System;
using MTV3D65;

namespace TroubleInEuclidea.Structure
{
    enum Direction
    {
        None,
        Up, 
        Right, 
        Down, 
        Left
    }

    static class DirectionExtensions
    {
        public static float AsTheta(this Direction direction)
        {
            switch (direction)
            {
                case Direction.Up: return -MathHelper.HalfPi;
                case Direction.Down: return MathHelper.HalfPi;
                case Direction.Left: return MathHelper.Pi;
                case Direction.Right: return 0;
            }
            throw new InvalidOperationException();
        }

        public static TV_2DVECTOR AsVector(this Direction direction)
        {
            switch (direction)
            {
                case Direction.Up: return new TV_2DVECTOR(0, 1);
                case Direction.Down: return new TV_2DVECTOR(0, -1);
                case Direction.Left: return new TV_2DVECTOR(-1, 0);
                case Direction.Right: return new TV_2DVECTOR(1, 0);
            }
            throw new InvalidOperationException();
        }

        public static Direction GetTangent(this Direction direction)
        {
            switch (direction)
            {
                case Direction.Up: return Direction.Right;
                case Direction.Down: return Direction.Left;
                case Direction.Left: return Direction.Up;
                case Direction.Right: return Direction.Down;
            }
            throw new InvalidOperationException();
        }

        public static Direction GetInverse(this Direction direction)
        {
            switch (direction)
            {
                case Direction.Up: return Direction.Down;
                case Direction.Down: return Direction.Up;
                case Direction.Left: return Direction.Right;
                case Direction.Right: return Direction.Left;
            }
            throw new InvalidOperationException();
        }
    }
}
