﻿using System.Collections.Generic;
using System;
using TroubleInEuclidea.Components.Enemies;

namespace TroubleInEuclidea.Structure
{
    enum EnemyType
    {
        Parallelogram,
        Triangle,
        Octogon,
        Circle,
        Square
    }

    static class EnemyTypeExtensions
    {
        public static Type GetClass(this EnemyType enemyType)
        {
            switch (enemyType)
            {
                //case EnemyType.Circle: 
                case EnemyType.Octogon: return typeof(OctogonEnemy);
                case EnemyType.Square: return typeof(SquareEnemy);
                case EnemyType.Parallelogram: return typeof(ParallelogramEnemy);
                case EnemyType.Triangle: return typeof(TriangleEnemy);
            }
            throw new InvalidOperationException();
        }

        public static IEnumerable<WaveType> GetSupportedWaveTypes(this EnemyType enemyType)
        {
            switch(enemyType)
            {
                case EnemyType.Parallelogram:
                    yield return WaveType.Burst;
                    yield return WaveType.Cluster;
                    yield return WaveType.Linear;
                    yield return WaveType.Random;
                    yield return WaveType.Square;
                    yield return WaveType.SynchronousBurst;
                    break;

                case EnemyType.Triangle:
                    yield return WaveType.Cluster;
                    yield return WaveType.Random;
                    yield return WaveType.Square;
                    break;

                case EnemyType.Circle:
                    yield return WaveType.Random;
                    break;

                case EnemyType.Octogon:
                    yield return WaveType.Burst;
                    yield return WaveType.Cluster;
                    yield return WaveType.Linear;
                    yield return WaveType.Random;
                    yield return WaveType.Square;
                    yield return WaveType.SynchronousBurst;
                break;
            }
        }

        public static int GetMaximumWaveSize(this EnemyType enemyType)
        {
            switch (enemyType)
            {
                //case EnemyType.Circle: return 1;
                case EnemyType.Triangle: return 8;
                case EnemyType.Octogon: return 6;
                case EnemyType.Parallelogram: return 18;
            }
            throw new InvalidOperationException();
        }

        public static double GetOccurence(this EnemyType enemyType)
        {
            switch (enemyType)
            {
                case EnemyType.Parallelogram: return 0.4;
                case EnemyType.Triangle: return 0.3;
                case EnemyType.Octogon: return 0.2;
                //case EnemyType.Circle: return 0.1;
            }
            throw new InvalidOperationException();
        }

        public static string GetSpawnSound(this EnemyType enemyType, WaveType waveType)
        {
            switch (enemyType)
            {
                case EnemyType.Parallelogram:
                   switch (waveType)
                   {
                       case WaveType.SynchronousBurst:
                       case WaveType.Linear:
                       case WaveType.Square:
                           return "buzz3";
                       default:
                           return "buzz2";
                   }

                case EnemyType.Triangle:
                   switch (waveType)
                   {
                       case WaveType.SynchronousBurst:
                       case WaveType.Linear:
                       case WaveType.Square:
                           return "buzz5";
                       default:
                           return "buzz9";
                   }

                case EnemyType.Octogon:
                   return "buzz4";
            }
            throw new InvalidOperationException();
        }
    }
}