﻿using System;
using MTV3D65;

namespace TroubleInEuclidea.Structure
{
    static class MathHelper
    {
        public const float Epsilon = 0.001f;
        public const float Pi = (float)Math.PI;
        public const float HalfPi = (float)(Math.PI / 2);
        public const float TwoPi = (float)(Math.PI * 2);

        public static float LookAt(double deltaY, double deltaX)
        {
            return LookAt(deltaY, deltaX, Direction.Right);
        }
        public static float LookAt(double deltaY, double deltaX, Direction initialDirection)
        {
            return initialDirection.AsTheta() - (float)Math.Atan2(deltaY, deltaX);
        }

        public static float CurveAngle(double from, double to, float step)
        {
            if (AlmostEqual(step, 0)) return (float)from;
            if (AlmostEqual(from, to) || AlmostEqual(step, 1)) return (float)to;

            var fromVector = new TV_2DVECTOR((float)Math.Cos(from), (float)Math.Sin(from));
            var toVector = new TV_2DVECTOR((float)Math.Cos(to), (float)Math.Sin(to));

            var currentVector = Slerp(fromVector, toVector, step);

            return (float)Math.Atan2(currentVector.y, currentVector.x);
        }

        public static bool AlmostEqual(this TV_2DVECTOR a, TV_2DVECTOR b, double epsilon)
        {
            return Math.Abs(a.x - b.x) <= epsilon && Math.Abs(a.y - b.y) <= epsilon;
        }
        public static bool AlmostEqual(this TV_2DVECTOR a, TV_2DVECTOR b, TV_2DVECTOR epsilon)
        {
            return Math.Abs(a.x - b.x) <= epsilon.x && Math.Abs(a.y - b.y) <= epsilon.y;
        }
        public static bool AlmostEqual(this TV_2DVECTOR a, TV_2DVECTOR b)
        {
            return Math.Abs(a.x - b.x) <= Epsilon && Math.Abs(a.y - b.y) <= Epsilon;
        }

        public static bool AlmostEqual(double a, double b, double epsilon)
        {
            return Math.Abs(a - b) <= epsilon;
        }
        public static bool AlmostEqual(double a, double b)
        {
            return Math.Abs(a - b) <= Epsilon;
        }

        public static TV_2DVECTOR Slerp(TV_2DVECTOR from, TV_2DVECTOR to, float step)
        {
            if (AlmostEqual(step, 0)) return from;
            if (AlmostEqual(step, 1)) return to;

            double theta = Math.Acos(from.Dot(to));
            if (AlmostEqual(theta, 0)) return to;

            double sinTheta = Math.Sin(theta);
            return (float)(Math.Sin((1 - step) * theta) / sinTheta) * from + (float)(Math.Sin(step * theta) / sinTheta) * to;
        }

        public static float WrapAngle(float theta)
        {
            while (theta >= Pi - Epsilon)
                theta -= TwoPi;
            while (theta < -Pi - Epsilon)
                theta += TwoPi;

            return theta;
        }

        public static float Clamp(float value, float minimum, float maximum)
        {
            return Math.Max(Math.Min(value, maximum), minimum);
        }

        public static float Lerp(float from, float to, float step)
        {
            return (to - from) * step + from;
        }
    }
}
