﻿using System;
using MTV3D65;

namespace TroubleInEuclidea.Structure
{
    static class SpriteFactory
    {
        readonly static TVScene Scene = new TVScene();
        readonly static TVTextureFactory TextureFactory = new TVTextureFactory();
        readonly static TVMaterialFactory MaterialFactory = new TVMaterialFactory();

        public static TVMesh CreateSprite(String textureName)
        {
            return CreateSprite(textureName, new TV_COLOR(1, 1, 1, 1));
        }
        public static TVMesh CreateSprite(String textureName, TV_COLOR color)
        {
            TVMesh mesh = Scene.CreateMeshBuilder();
            mesh.AddWall(TextureFactory.LoadTexture(textureName + ".dds", textureName), 0, 0, 1, 0, 1);
            mesh.SetMeshCenter(0.5f, 0.5f, 0);
            mesh.SetCullMode(CONST_TV_CULLING.TV_DOUBLESIDED);
            mesh.SetBlendingMode(CONST_TV_BLENDINGMODE.TV_BLEND_ADDALPHA);
            mesh.SetAlphaTest(false);
            mesh.SetLightingMode(CONST_TV_LIGHTINGMODE.TV_LIGHTING_NORMAL);

            int mat = MaterialFactory.CreateMaterial();
            MaterialFactory.SetAmbient(mat, color.r, color.g, color.b, color.a);
            MaterialFactory.SetDiffuse(mat, color.r, color.g, color.b, color.a);
            MaterialFactory.SetEmissive(mat, color.r, color.g, color.b, color.a);
            mesh.SetMaterial(mat);

            return mesh;
        }
    }
}
