﻿using System;
using MTV3D65;

namespace TroubleInEuclidea.Structure
{
    static class Vector2
    {
        public static TV_2DVECTOR One
        {
            get { return new TV_2DVECTOR(1, 1); }
        }
        public static TV_2DVECTOR Zero
        {
            get { return new TV_2DVECTOR(); }
        }
        public static TV_2DVECTOR UnitX
        {
            get { return new TV_2DVECTOR(1, 0); }
        }
        public static TV_2DVECTOR UnitY
        {
            get { return new TV_2DVECTOR(0, 1); }
        }
        public static TV_2DVECTOR Up
        {
            get { return new TV_2DVECTOR(0, -1); }
        }
        public static TV_2DVECTOR Down
        {
            get { return new TV_2DVECTOR(0, 1); }
        }
        public static TV_2DVECTOR Left
        {
            get { return new TV_2DVECTOR(-1, 0); }
        }
        public static TV_2DVECTOR Right
        {
            get { return new TV_2DVECTOR(1, 0); }
        }

        public static float Dot(this TV_2DVECTOR a, TV_2DVECTOR b)
        {
            return a.x * b.x + a.y * b.y;
        }

        public static TV_2DVECTOR Normalize(this TV_2DVECTOR vector)
        {
            float length = vector.Length();
            return length == 0 ? Zero : vector / length;
        }

        public static TV_2DVECTOR ComponentWiseProduct(this TV_2DVECTOR a, TV_2DVECTOR b)
        {
            return new TV_2DVECTOR(a.x * b.x, a.y * b.y);
        }

        public static float Length(this TV_2DVECTOR vector)
        {
            return (float)Math.Sqrt(vector.x * vector.x + vector.y * vector.y);
        }

        public static TV_2DVECTOR Clamp(this TV_2DVECTOR vector, TV_2DVECTOR reflectiveBound)
        {
            return vector.Clamp(-1 * reflectiveBound, reflectiveBound);
        }

        public static TV_2DVECTOR Clamp(this TV_2DVECTOR vector, TV_2DVECTOR minimum, TV_2DVECTOR maximum)
        {
            return new TV_2DVECTOR(
                Math.Min(Math.Max(vector.x, minimum.x), maximum.x),
                Math.Min(Math.Max(vector.y, minimum.y), maximum.y));
        }

        public static bool IsInside(this TV_2DVECTOR vector, TV_2DVECTOR reflectiveBound)
        {
            return vector.x >= -reflectiveBound.x && vector.x <= reflectiveBound.x &&
                vector.y >= -reflectiveBound.y && vector.y <= reflectiveBound.y;
        }

        public static TV_2DVECTOR Lerp(this TV_2DVECTOR from, TV_2DVECTOR to, double step)
        {
            return new TV_2DVECTOR(MathHelper.Lerp(from.x, to.x, (float)step), MathHelper.Lerp(from.y, to.y, (float)step));
        }

        public static TV_2DVECTOR Abs(this TV_2DVECTOR vector)
        {
            return new TV_2DVECTOR(Math.Abs(vector.x), Math.Abs(vector.y));
        }

        public static TV_3DVECTOR ToVector3(this TV_2DVECTOR vector)
        {
            return new TV_3DVECTOR(vector.x, vector.y, 0);
        }

        public static TV_2DVECTOR ToVector2(this TV_3DVECTOR vector)
        {
            return new TV_2DVECTOR(vector.x, vector.y);
        }
    }
}
