﻿using System;
using System.Collections.Generic;
using TroubleInEuclidea.Components.Enemies;
using TroubleInEuclidea.Components.Waves;

namespace TroubleInEuclidea.Structure
{
    enum WaveType
    {
        Burst,
        Cluster,
        Linear,
        Random,
        Square,
        SynchronousBurst
    }

    static class WaveTypeExtensions
    {
        public static Type GetClass(this WaveType waveType, EnemyType enemyType)
        {
            switch (waveType)
            {
                case WaveType.Burst: return ReflectionHelper.MakeGenericType<BurstWave<Enemy>>(enemyType.GetClass());
                case WaveType.Cluster: return ReflectionHelper.MakeGenericType<ClusterWave<Enemy>>(enemyType.GetClass());
                case WaveType.Linear: return ReflectionHelper.MakeGenericType<LinearWave<Enemy>>(enemyType.GetClass());
                case WaveType.Random: return ReflectionHelper.MakeGenericType<RandomWave<Enemy>>(enemyType.GetClass());
                case WaveType.Square: return ReflectionHelper.MakeGenericType<SquareWave<Enemy>>(enemyType.GetClass());
                case WaveType.SynchronousBurst: return ReflectionHelper.MakeGenericType<SynchronousBurstWave<Enemy>>(enemyType.GetClass());
            }
            throw new InvalidOperationException();
        }

        public static IEnumerable<WaveType> GetSimultaneousWaves(this WaveType waveType)
        {
            switch (waveType)
            {
                case WaveType.SynchronousBurst:
                case WaveType.Burst:
                    yield return WaveType.Cluster;
                    yield return WaveType.Random;
                    yield return WaveType.Square;
                    yield return WaveType.Linear;
                    break;

                case WaveType.Random:
                case WaveType.Cluster:
                    yield return WaveType.Cluster;
                    yield return WaveType.Random;
                    yield return WaveType.Square;
                    yield return WaveType.Linear;
                    yield return WaveType.SynchronousBurst;
                    yield return WaveType.Burst;
                    break;

                case WaveType.Linear:
                case WaveType.Square: 
                    yield return WaveType.Burst;
                    yield return WaveType.Cluster;
                    yield return WaveType.Random;
                    break;
            }
            throw new InvalidOperationException();
        }

        public static double GetOccurence(this WaveType waveType)
        {
            switch (waveType)
            {
                case WaveType.Cluster: return 0.25;
                case WaveType.Linear: return 0.2;
                case WaveType.Burst: return 0.175;
                case WaveType.Random: return 0.15;
                case WaveType.Square: return 0.125;
                case WaveType.SynchronousBurst: return 0.1;

            }
            throw new InvalidOperationException();
        }

        public static double GetWaveCountModifier(this WaveType waveType)
        {
            switch (waveType)
            {
                case WaveType.Random: 
                case WaveType.Cluster: 
                case WaveType.Linear:
                    return 1;
                case WaveType.Burst: 
                case WaveType.Square:
                    return 2;
                case WaveType.SynchronousBurst:
                    return 2;
            }
            throw new InvalidOperationException();
        } 
    }
}
